@@ -1065,6 +1065,7 @@ bit_write_BE (Bit_Chain *dat, double x, double y, double z)
     }
 }
 
+// TODO ensure length 1, dwg_geom_normalize()
 void
 normalize_BE (BITCODE_3BD ext)
 {
@@ -1542,8 +1543,9 @@ static int heX (unsigned char c)
   return c >= 10 ? 'A' + c - 10 : '0' + c;
 }
 
-/* converts TU to ASCII with embedded \U+XXXX.
-   max len widechars.
+/* Converts TU to ASCII with embedded \U+XXXX.
+   Max len widechars.
+   No codepage support yet, only cp 30.
  */
 char *
 bit_embed_TU_size (BITCODE_TU restrict wstr, const int len)
@@ -1813,7 +1815,7 @@ bit_write_T (Bit_Chain *restrict dat, BITCODE_T restrict s)
 {
   int i, length;
 
-  // only if from r2007+ DWG, not JSON, DXF
+  // only if from r2007+ DWG, not JSON, DXF, add API
   if (IS_FROM_TU (dat))
     {
       // downconvert TU to TV
@@ -2200,10 +2202,12 @@ bit_convert_TU (BITCODE_TU restrict wstr)
       while (c)
         {
           len++;
-          if (c >= 256)
-            len++;
-          if (c >= 0x800)
-            len++;
+          if (c >= 0x80)
+            {
+              len++;
+              if (c >= 0x800)
+                len++;
+            }
           b += 2;
           c = (b[0] << 8) + b[1];
         }
@@ -2213,16 +2217,18 @@ bit_convert_TU (BITCODE_TU restrict wstr)
   while ((c = *tmp++))
     {
       len++;
-      if (c >= 256)
-        len++;
-      if (c >= 0x800)
-        len++;
+      if (c >= 0x80)
+        {
+          len++;
+          if (c >= 0x800)
+            len++;
+        }
 #if 0
         loglevel = 5;
         LOG_INSANE ("U+%04X ", c);
 #endif
     }
-  str = (char*)malloc (len + 1);
+  str = (char*)malloc (len + 2);
   if (!str)
     {
       loglevel = 1;
@@ -2235,9 +2241,9 @@ bit_convert_TU (BITCODE_TU restrict wstr)
     {
       unsigned char *b = (unsigned char*)wstr;
       c = (b[0] << 8) + b[1];
-      while (c)
+      while (c && i < len)
         {
-          if (c < 256)
+          if (c < 0x80)
             {
               str[i++] = c & 0xFF;
             }
@@ -2259,9 +2265,9 @@ bit_convert_TU (BITCODE_TU restrict wstr)
     }
   else
 #endif
-  while ((c = *wstr++))
+  while ((c = *wstr++) && i < len)
     {
-      if (c < 256)
+      if (c < 0x80)
         {
           str[i++] = c & 0xFF;
         }
@@ -2292,7 +2298,8 @@ bit_convert_TU (BITCODE_TU restrict wstr)
         HANDLER (OUTPUT, "ERROR: overlarge unicode codepoint U+%0X", c);
      */
     }
-  str[i] = '\0';
+  if (i <= len + 1)
+    str[i] = '\0';
   return str;
 }
 
@@ -2302,7 +2309,8 @@ bit_convert_TU (BITCODE_TU restrict wstr)
     Returns NULL if not enough room in dest.
 */
 char *
-bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src, const int destlen)
+bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src,
+                const int destlen, const unsigned cquoted)
 {
   unsigned char c;
   unsigned char *s = (unsigned char *)src;
@@ -2315,7 +2323,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src, const in
         {
           return NULL;
         }
-      else if (c == '\\' && dest+1 < endp &&
+      else if (cquoted && c == '\\' && dest+1 < endp &&
           // skip \" to " and \\ to \.
           (*s == '"' || *s == '\\' || *s == 'r' || *s == 'n'))
         {
@@ -2406,7 +2414,7 @@ bit_utf8_to_TV (char *restrict dest, const unsigned char *restrict src, const in
     TODO: unquote json_cquote as above.
  */
 BITCODE_TU
-bit_utf8_to_TU (char *restrict str)
+bit_utf8_to_TU (char *restrict str, const unsigned cquoted)
 {
   BITCODE_TU wstr;
   int i = 0;
@@ -2464,12 +2472,12 @@ bit_utf8_to_TU (char *restrict str)
 
 /* compare an ASCII/TU string to ASCII name */
 int
-bit_eq_T (Bit_Chain *restrict dat, const BITCODE_T restrict str1, const char *restrict str2)
+bit_eq_T (Bit_Chain *restrict dat, const BITCODE_T restrict wstr1, const char *restrict str2)
 {
-  if (dat->version >= R_2007)
-    return bit_eq_TU (str2, (BITCODE_TU)str1);
+  if (IS_FROM_TU (dat))
+    return bit_eq_TU (str2, (BITCODE_TU)wstr1);
   else
-    return !strcmp (str1, str2);
+    return !strcmp (wstr1, str2);
 }
 
 /* compare an ASCII/utf-8 string to a r2007+ name */
@@ -2512,7 +2520,7 @@ bit_set_T (Bit_Chain *dat, const char* restrict src)
   if (!(IS_FROM_TU (dat)))
     return strdup (src);
   else
-    return (BITCODE_T)bit_utf8_to_TU ((char*)src);
+    return (BITCODE_T)bit_utf8_to_TU ((char*)src, 0);
 }
 
 /** Read 1 bitlong according to normal order
