@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009, 2018 Free Software Foundation, Inc.                  */
+/*  Copyright (C) 2009-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -24,8 +24,22 @@
 #include "bits.h"
 #include "dwg.h"
 
-int dwg_encode (Dwg_Data *restrict dwg, Bit_Chain *restrict dat);
-uint32_t dwg_section_page_checksum (const uint32_t seed, Bit_Chain *dat,
-                                    uint32_t size);
+/* only when the versions mismatch */
+#define ENCODE_UNKNOWN_AS_DUMMY
+
+#ifdef IN_DXF_H
+#error in_dxf.h must be included after __FILE__ because of FORMAT_BD
+#endif
+
+EXPORT int dwg_encode (Dwg_Data *restrict dwg, Bit_Chain *restrict dat);
+
+/** Returns the class for the object.
+    First searching class by name, not offset
+    because indxf has a different class order. Fixes up the obj->type then.
+    Also searches via dxfname aliases, like DICTIONARYWDFLT => ACDBDICTIONARYWDFLT
+    and fixes up obj->type also then.
+    If obj->dxfname is NULL, the search is done by index, without any fixups.
+ */
+Dwg_Class *dwg_encode_get_class (Dwg_Data *restrict dwg, Dwg_Object *restrict obj);
 
 #endif
