@@ -11,7 +11,7 @@
 /*****************************************************************************/
 
 /*
- * in_json.h: read json to dwg
+ * in_json.h: read json to dwg (draft, WIP)
  * written by Reini Urban
  */
 
@@ -21,7 +21,12 @@
 #include "dwg.h"
 #include "bits.h"
 
-int dwg_read_json (Bit_Chain *dat, Dwg_Data *dwg);
-void dwg_json_object (Bit_Chain *dat, Dwg_Object *obj);
+#ifdef IN_DXF_H
+#error in_dxf.h must be included after in_json.h because of FORMAT_BD
+#endif
+
+EXPORT int dwg_read_json (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
+//TODO needed? See [GH #136]
+//void dwg_injson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj);
 
 #endif
