@@ -373,6 +373,9 @@
 #  define FIELD_HANDLE0(name, code, dxf) FIELD_HANDLE (name, code, dxf)
 #  define SUB_FIELD_HANDLE0(o, name, code, dxf) SUB_FIELD_HANDLE (o, name, code, dxf)
 #endif
+#ifndef VALUE_TV0
+#  define VALUE_TV0(name, dxf) VALUE_TV (name, dxf)
+#endif
 
 // double to text
 #ifndef FIELD_D2T
@@ -441,12 +444,11 @@
 #  undef SET_PARENT_OBJ
 #  undef SET_PARENT_FIELD
 #  define SET_PARENT(field, to)                                               \
-    if (_obj->field)                                                          \
-      _obj->field->parent = to;
-#  define SET_PARENT_OBJ(field) SET_PARENT (field, _obj);
+    _obj->field.parent = to
+#  define SET_PARENT_OBJ(field)                                               \
+    SET_PARENT (field, _obj)
 #  define SET_PARENT_FIELD(field, what_parent, to)                            \
-    if (_obj->field)                                                          \
-      _obj->field->what_parent = to;
+    _obj->field.what_parent = to
 #else
 #  define TRACE_DD
 #endif
