@@ -1,6 +1,7 @@
 #define BITS_TEST_C
-#include "../../src/bits.h"
-#include "../../src/common.h"
+#include "config.h"
+#include "bits.h"
+#include "common.h"
 #include <string.h>
 #include <stdlib.h>
 #include "tests_common.h"
@@ -327,6 +328,7 @@ main (int argc, char const *argv[])
   unsigned char sentinel[]
       = { 0xCF, 0x7B, 0x1F, 0x23, 0xFD, 0xDE, 0x38, 0xA9,
           0x5F, 0x7C, 0x68, 0xB8, 0x4E, 0x6D, 0x33, 0x5F };
+  loglevel = is_make_silent() ? 0 : 3;
 
   bit_read_B_tests ();
   bit_write_B_tests ();
